dnl Macros for configuring WHIZARD subdirectories and subpackages
dnl All macros begin with 'WK'

dnl Horizontal line for readability:
AC_DEFUN(WK_HLINE,
[AC_MSG_NOTICE([--------------------------------------------------------------])])

dnl Message at the beginning of a configure section
AC_DEFUN(WK_CONFIGURE_SECTION,
[WK_HLINE()
AC_MSG_NOTICE([--- ]$1[ ---])
AC_MSG_NOTICE([])
])


dnl Define a variable and export it
dnl WK_WHIZARD_SET(variable, value)
AC_DEFUN(WK_WHIZARD_SET,
[$1=$2
AC_SUBST($1)])

dnl Add a list of names to the list of output resp. executable files
dnl WK_WHIZARD_OUTFILES(subdir, files)
AC_DEFUN(WK_WHIZARD_OUTFILES,
[for file in $2; do
  OUTFILES="$OUTFILES $1/$file";
done])

dnl WK_WHIZARD_EXECUTABLES(subdir, files)
AC_DEFUN(WK_WHIZARD_EXECUTABLES,
[for file in $2; do
  OUTFILES="$OUTFILES $1/$file";
  EXECUTABLES="$EXECUTABLES $1/$file"
done])


dnl This adds a package which resides in a WHIZARD subdirectory.
dnl The `variable' is inserted into AC_SUBST; it will refer to 
enl the package subdirectory in Makefiles.  The package
dnl has a subdirectory XXX-src and optionally a Makefile (or similar).
dnl WK_WHIZARD_PACKAGE(variable, identifier [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE,
[WK_WHIZARD_SET($1,$2-src)
ifelse($#, 3, 
[WK_WHIZARD_OUTFILES($$1, $3)])
ifelse($#, 4, 
[WK_WHIZARD_OUTFILES($$1, $3)
WK_WHIZARD_EXECUTABLES($$1, $4)])
])

dnl This is similar, but no '-src' is appended for the directory name
dnl WK_WHIZARD_SUBDIR(variable, identifier [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_SUBDIR,
[WK_WHIZARD_SET($1,$2)
ifelse($#, 3,
[WK_WHIZARD_OUTFILES($$1, $3)])
ifelse($#, 4, 
[WK_WHIZARD_OUTFILES($$1, $3)
WK_WHIZARD_EXECUTABLES($$1, $4)])
])


dnl This adds a matrix element generating package.
dnl There must be at least one Makefile.
dnl In addition to the preceding macro, it adds two scripts
dnl init.XXX and make-proc.XXX where XXX is the package identifier.
dnl WK_WHIZARD_ME_GENERATOR(variable, identifier, Makefile)
AC_DEFUN(WK_WHIZARD_ME_GENERATOR,
[WK_WHIZARD_PACKAGE($1,$2,$3, init.$2 make-proc.$2)])


dnl This is like WK_WHIZARD_PACKAGE, but it calls AC_ARG_ENABLE in addition.
dnl If the package `id' is disabled or the 'file' is not found,
dnl the variable `var' is set to "no".
dnl WK_WHIZARD_PACKAGE_ENABLE(var, id, help, file [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE_ENABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "no"; then
AC_MSG_CHECKING([for $2-src/$4])
WK_WHIZARD_SET($1, no)
AC_MSG_RESULT([(disabled)])
else
AC_CHECK_FILE($2-src/$4, enable_$2=yes, enable_$2=no)
if test "$enable_$2" = "no"; then
WK_WHIZARD_SET($1, no)
else
ifelse($#, 4, [WK_WHIZARD_PACKAGE($1, $2)])
ifelse($#, 5, [WK_WHIZARD_PACKAGE($1, $2, $5)])
ifelse($#, 6, [WK_WHIZARD_PACKAGE($1, $2, $5, $6)])
fi
fi
])

dnl The same, but disabled by default.
dnl If the package `id' is disabled or the 'file' is not found,
dnl the variable `var' is set to "no".
dnl WK_WHIZARD_PACKAGE_DISABLE(var, id, help, file [,Makefiles [,Executables]])
AC_DEFUN(WK_WHIZARD_PACKAGE_DISABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "yes"; then
AC_CHECK_FILE($2-src/$4, enable_$2=yes, enable_$2=no)
if test "$enable_$2" = "no"; then
WK_WHIZARD_SET($1, no)
else
ifelse($#, 4, [WK_WHIZARD_PACKAGE($1, $2)])
ifelse($#, 5, [WK_WHIZARD_PACKAGE($1, $2, $5)])
ifelse($#, 6, [WK_WHIZARD_PACKAGE($1, $2, $5, $6)])
fi
else
enable_$2="no"
AC_MSG_CHECKING([for $2-src/$4])
WK_WHIZARD_SET($1, no)
AC_MSG_RESULT([(disabled)])
fi
])

dnl The same for a matrix element generator
dnl There must be at least one Makefile.
dnl WK_WHIZARD_ME_GENERATOR_ENABLE(var, id, help, file, Makefiles [,Executables])
AC_DEFUN(WK_WHIZARD_ME_GENERATOR_ENABLE,
[WK_WHIZARD_PACKAGE_ENABLE($1, $2, [$3], $4, $5, init.$2 make-proc.$2 $5)])

dnl The same, disabled by default:
dnl WK_WHIZARD_ME_GENERATOR_DISABLE(var, id, help, file, Makefiles [,Executables])
AC_DEFUN(WK_WHIZARD_ME_GENERATOR_DISABLE,
[WK_WHIZARD_PACKAGE_DISABLE($1, $2, [$3], $4, $5, init.$2 make-proc.$2 $5)])



dnl ------------------------------------------------------------------------
dnl Fortran compiler tests

dnl This looks for a Fortran 95 compiler.
dnl The standard Fortran compiler test is AC_PROG_FC.
dnl We use this, but require the .f90 extension for free-form code.
dnl At the end FC, FCFLAGS and FCFLAGS_f90 are set, if successful.
AC_DEFUN(WK_PROG_FC_F95,
[AC_FC_SRCEXT(f90)
AC_PROG_FC(,[Fortran 95])
])


dnl Determine vendor and version string.
AC_DEFUN(WK_FC_GET_VENDOR,
[AC_CACHE_CHECK([the compiler ID],
[wk_cv_prog_f90_version_string],
[$FC -version >conftest.log 2>&1
$FC -V >>conftest.log 2>&1
$FC --version >>conftest.log 2>&1

wk_grep_f90_NAG=`grep NAG conftest.log | head -1`
wk_grep_f90_Compaq=`grep Compaq conftest.log | head -1`
wk_grep_f90_Digital=`grep DIGITAL conftest.log | head -1`
wk_grep_f90_SGI=`grep MIPS conftest.log | head -1`
wk_grep_f90_Intel=`grep 'Intel(R)' conftest.log | head -1`
wk_grep_f90_Sun=`grep 'Sun' conftest.log | head -1`
wk_grep_f90_Lahey=`grep 'Lahey' conftest.log | head -1`
wk_grep_f90_PGI=`grep 'pgf' conftest.log | head -1`
wk_grep_f90_G95=`grep -i 'g95' conftest.log | grep -i 'gcc' | head -1`
wk_grep_f90_GFORTRAN=`grep -i 'GNU Fortran' conftest.log | head -1`
 
if test -n "$wk_grep_f90_NAG"; then
  wk_cv_prog_f90_type="NAG"
  wk_cv_prog_f90_version_string=$wk_grep_f90_NAG
  wk_cv_prog_f90_version=[`echo $wk_cv_prog_f90_version_string | sed -e 's/.* Release \([0-9][0-9]*\.[0-9][0-9]*.*$\)/\1/'`]
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/\([0-9][0-9]*\)\..*/\1/'`]
elif test -n "$wk_grep_f90_Compaq"; then
  wk_cv_prog_f90_type="Compaq"
  wk_cv_prog_f90_version_string=$wk_grep_f90_Compaq
elif test -n "$wk_grep_f90_Digital"; then
  wk_cv_prog_f90_type="DEC"
  wk_cv_prog_f90_version_string=$wk_grep_f90_Digital
elif test -n "$wk_grep_f90_SGI"; then
  wk_cv_prog_f90_type="SGI"
  wk_cv_prog_f90_version_string=$wk_grep_f90_SGI
elif test -n "$wk_grep_f90_Intel"; then
  wk_cv_prog_f90_type="Intel"
  wk_cv_prog_f90_version_string=$wk_grep_f90_Intel
  wk_cv_prog_f90_version=[`echo $wk_cv_prog_f90_version_string | sed -e 's/.* Version \([0-9][0-9]*\.[0-9][0-9]*\) .*/\1/'`]
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/\([0-9][0-9]*\)\..*/\1/'`]
elif test -n "$wk_grep_f90_Sun"; then
  wk_cv_prog_f90_type="Sun"
  wk_cv_prog_f90_version_string=$wk_grep_f90_Sun
  wk_cv_prog_f90_version=[`echo $wk_cv_prog_f90_version_string | sed -e 's/.* Fortran 95 \([0-9][0-9]*\.[0-9][0-9]*\) .*/\1/'`]
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/\([0-9][0-9]*\)\..*/\1/'`]
elif test -n "$wk_grep_f90_Lahey"; then
  wk_cv_prog_f90_type="Lahey"
  wk_cv_prog_f90_version_string=$wk_grep_f90_Lahey
elif test -n "$wk_grep_f90_PGI"; then
  wk_cv_prog_f90_type="PGI"
  wk_cv_prog_f90_version_string=$wk_grep_f90_PGI
elif test -n "$wk_grep_f90_G95"; then
  wk_cv_prog_f90_type="G95"
  wk_cv_prog_f90_version_string=$wk_grep_f90_G95
elif test -n "$wk_grep_f90_GFORTRAN"; then
  wk_cv_prog_f90_type="GNU"
  wk_cv_prog_f90_version_string=$wk_grep_f90_GFORTRAN
  wk_cv_prog_f90_version=[`echo $wk_cv_prog_f90_version_string | sed -e 's/.*\([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\).*/\1/'`]
  wk_cv_prog_f90_major_version=[`echo $wk_cv_prog_f90_version | sed -e 's/\([0-9][0-9]*\)\..*/\1/'`]
else
  wk_cv_prog_f90_type="unknown"
  wk_cv_prog_f90_version_string="unknown"
fi

rm -f conftest.log

])  dnl end AC_CACHE_CHECK


dnl Vendor-specific variables:
AC_CACHE_CHECK([the compiler vendor], [wk_cv_prog_f90_type])

if test -n "$wk_cv_prog_f90_version"; then
  AC_CACHE_CHECK([the compiler version], [wk_cv_prog_f90_version])
else
  wk_cv_prog_f90_version=$wk_cv_prog_f90_version_string
fi

if test -n "$wk_cv_prog_f90_major_version"; then
  AC_CACHE_CHECK([the compiler major version], [wk_cv_prog_f90_major_version])
else
  wk_cv_prog_f90_major_version=$wk_cv_prog_f90_version
fi

FC_VERSION_STRING=$wk_cv_prog_f90_version_string
FC_VENDOR=$wk_cv_prog_f90_type
FC_VERSION=$wk_cv_prog_f90_version
FC_MAJOR_VERSION=$wk_cv_prog_f90_major_version
AC_SUBST(FC_VERSION_STRING)
AC_SUBST(FC_VENDOR)
AC_SUBST(FC_VERSION)
AC_SUBST(FC_MAJOR_VERSION)

dnl Module names: (all compilers apparently have converged to '.mod')
dnl The perl scripts need a quoted version of this
FC_MODNAME='$(1:.o=.mod)'
FC_MODNAME_Q='\$(1:.o=.mod)'
AC_SUBST(FC_MODNAME)
AC_SUBST(FC_MODNAME_Q)

])  dnl end AC_DEFUN


dnl Check for basic F95 features
AC_DEFUN(WK_FC_CHECK_F95,
[AC_CACHE_CHECK([whether this compiler supports Fortran 95 features],
[wk_cv_prog_f95_standard],
[AC_LANG(Fortran)
AC_COMPILE_IFELSE([
program conftest
   integer, dimension(2) :: ii
   integer :: i
   type :: foo
      integer, pointer :: bar => null ()
   end type foo
   forall (i = 1:2)  ii(i) = i
  contains
   elemental function f(x)
     real, intent(in) :: x
     real :: f
     f = x
   end function f
 end program conftest
],
wk_cv_prog_f95_standard=yes,
[AC_MSG_RESULT(no)
AC_MSG_FAILURE([Compiler does not support the Fortran 95 standard.])
])
FC_SUPPORTS_F95=$wk_cv_prog_f95_standard
AC_SUBST(FC_SUPPORTS_F95)
])
])
 

dnl Check for the TR15581 extensions (allocatable subobjects)
AC_DEFUN(WK_FC_CHECK_TR15581,
[AC_CACHE_CHECK([whether this compiler supports allocatable subobjects],
[wk_cv_prog_f95_allocatable],
[AC_LANG(Fortran)
AC_COMPILE_IFELSE([
program conftest
  type :: foo
     integer, dimension(:), allocatable :: bar
  end type foo
end program conftest
],
wk_cv_prog_f95_allocatable=yes,
wk_cv_prog_f95_allocatable=no)
FC_SUPPORTS_ALLOCATABLE=$wk_cv_prog_f95_allocatable
AC_SUBST(FC_SUPPORTS_ALLOCATABLE)
])
])

dnl Check for the C bindings extensions of Fortran 2003
AC_DEFUN(WK_FC_CHECK_C_BINDING,
[AC_CACHE_CHECK([whether this compiler supports C binding (Fortran 2003)],
[wk_cv_prog_f95_c_binding],
[AC_LANG(Fortran)
AC_COMPILE_IFELSE([
program conftest
  use iso_c_binding
  type, bind(c) :: t
     integer(c_int) :: i
     real(c_float) :: x
  end type t
end program conftest
],
wk_cv_prog_f95_c_binding=yes,
wk_cv_prog_f95_c_binding=no)
FC_SUPPORTS_ALLOCATABLE=$wk_cv_prog_f95_c_binding
AC_SUBST(FC_SUPPORTS_C_BINDING)
])
])


dnl Check for the command line interface of Fortran 2003
dnl We actually have to link in order to check availability
AC_DEFUN(WK_FC_CHECK_CMDLINE,
[AC_CACHE_CHECK([whether this compiler interfaces the command line (F2003)],
[wk_cv_prog_f95_cmdline],
[AC_LANG(Fortran)
AC_LINK_IFELSE([
program conftest
  call get_command_argument (command_argument_count ())
end program conftest
],
wk_cv_prog_f95_cmdline=yes,
wk_cv_prog_f95_cmdline=no)
FC_SUPPORTS_CMDLINE=$wk_cv_prog_f95_cmdline
AC_SUBST(FC_SUPPORTS_CMDLINE)
])
])


dnl Check for quadruple precision support (real and complex!)
dnl If quadruple is actually required but not supported, report failure.
AC_DEFUN(WK_FC_CHECK_QUADRUPLE,
[AC_CACHE_CHECK([whether this compiler permits quadruple real and complex],
[wk_cv_prog_f95_quadruple],
[AC_LANG(Fortran)
AC_COMPILE_IFELSE([
program conftest
   integer, parameter :: d=selected_real_kind(precision(1.)+1, range(1.)+1)
   integer, parameter :: q=selected_real_kind(precision(1._d)+1, range(1._d))
   real(kind=q) :: x
   complex(kind=q) :: z
end program conftest
], 
wk_cv_prog_f95_quadruple=yes,
if test "$PRECISION" = "quadruple"; then
[AC_MSG_RESULT(no)
AC_MSG_FAILURE([Quadruple precision requested but not supported])]
else
wk_cv_prog_f95_quadruple=no
fi)
])
FC_SUPPORTS_QUADRUPLE=$wk_cv_prog_f95_quadruple
AC_SUBST(FC_SUPPORTS_QUADRUPLE)
if test "$FC_SUPPORTS_QUADRUPLE" = "yes"; then
  FC_QUAD_OR_SINGLE="quadruple"
else
  FC_QUAD_OR_SINGLE="single"
fi
AC_SUBST(FC_QUAD_OR_SINGLE)
])


dnl Check for profiling support
AC_DEFUN(WK_FC_CHECK_PROFILING,
[AC_CACHE_CHECK([whether this compiler supports profiling via -pg],
[wk_cv_prog_fc_profiling],
[AC_LANG(Fortran)
fcflags_tmp=$FCFLAGS
FCFLAGS="-pg $FCFLAGS"
rm -f gmon.out
AC_RUN_IFELSE([
program conftest
end program conftest
],
if test -f gmon.out; then
  wk_cv_prog_fc_profiling=yes
else
  wk_cv_prog_fc_profiling=no
fi,
wk_cv_prog_fc_profiling=no,
wk_cv_prog_fc_profiling="maybe [cross-compiling]")
rm -f gmon.out
FCFLAGS=$fcflags_tmp
FC_SUPPORTS_PROFILING=$wk_cv_prog_fc_profiling
AC_SUBST(FC_SUPPORTS_PROFILING)
])
])


dnl ------------------------------------------------------------------------
dnl Determine runtime libraries for additional Fortran compilers
dnl Since the results would interfere with the settings for the
dnl default Fortran compiler, do this by separate configure scripts
dnl in a subdirectory.

dnl WK_FC_LIBRARY_LDFLAGS(compiler-list)
dnl Take a list of Fortran compilers and determine the runtime libraries
dnl needed for them.  The result is concatenated to make up FCLIBS
AC_DEFUN(WK_FC_LIBRARY_LDFLAGS,
[if test -n "$1"; then
AC_MSG_CHECKING([for extra Fortran runtime libraries])
AC_MSG_RESULT(($1))
for comp in $1; do
  cd config.scripts
  sh configure.flibs FC=$comp
  wk_fclibs=`cat configure.flibs.out`
  FCLIBS="$FCLIBS $wk_fclibs"
  cd ..
done
fi
])



dnl ------------------------------------------------------------------------
dnl Fortran library tests

dnl Extension of AC_PATH_PROG: Search for libraries for which the name
dnl is not exactly known (because it may have the version number in it)
dnl Set output variables $var, $var_DIR, $var_LIB accordingly
dnl WHIZARD_PATH_LIB(var, id, name-list, path)
AC_DEFUN(WHIZARD_PATH_LIB,
[AC_CACHE_CHECK(for $3, wk_cv_path_$1,
[case "$$1" in
  /*)
  wk_cv_path_$1="$$1" # User-supplied path
  ;;
  *)
  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
  ac_dummy=$4
  for ac_dir in $ac_dummy; do
    test -z "$ac_dir" && ac_dir=.
    unset wk_cv_path_$1
    ac_pwd=`pwd`
    if test -d "$ac_dir"; then
      cd $ac_dir
      for ac_word in $3; do
        test -f "$ac_word" && wk_cv_path_$1="$ac_dir/$ac_word"
      done
      cd $ac_pwd
    fi
    if test -n "$wk_cv_path_$1"; then
      break
    fi
  done
  IFS="$ac_save_ifs"
  if test -z "$wk_cv_path_$1"; then
    wk_cv_path_$1="no"
  fi
  ;;
esac
])
$1=$wk_cv_path_$1
if test "$$1" != "no"; then
$1_DIR=`echo $$1 | sed -e 's|/lib$2.*\.a$||'`
$1_LIB=`echo $$1 | sed -e 's|^.*/lib\($2.*\)\.a$|\1|'`
fi
AC_SUBST($1)
AC_SUBST($1_DIR)
AC_SUBST($1_LIB)
])


dnl Library search with enable option, enabled by default
dnl WHIZARD_LIB_ENABLE(var, id, help, name-list, path)
AC_DEFUN(WHIZARD_LIB_ENABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "no"; then
AC_MSG_CHECKING(for $4)
$1="no"
AC_MSG_RESULT([(disabled)])
else
WHIZARD_PATH_LIB($1, $2, $4, $5)
fi
])


dnl Library search with enable option, disabled by default
dnl WHIZARD_LIB_DISABLE(var, id, help, name-list, path)
AC_DEFUN(WHIZARD_LIB_DISABLE,
[AC_ARG_ENABLE($2,[$3])
if test "$enable_$2" = "yes"; then
WHIZARD_PATH_LIB($1, $2, $4, $5)
else
AC_MSG_CHECKING(for $4)
$1="no"
AC_MSG_RESULT([(disabled)])
fi
])

dnl Library search depending on another feature being enabled
dnl WHIZARD_LIB_IF_ENABLED(var, id, enable_var, name-list, path)
AC_DEFUN(WHIZARD_LIB_IF_ENABLED,
[if test "$3" != "no"; then
WHIZARD_PATH_LIB($1, $2, $4, $5)
fi
])


dnl Check whether the Fortran compiler can link to a library.
dnl We try to call a subroutine that is defined by the library.
dnl If successful, prepend the library(s) in question to LIBS.
dnl WHIZARD_CHECK_LIB_FC(lib, lib-basename, lib-dir, fun, extra-libs)
AC_DEFUN(WHIZARD_CHECK_LIB_FC,
[if test "$$1" != "no"; then
wk_otherlibs="-L$3 $5"
AC_LANG(Fortran)
AC_CHECK_LIB($2,$4,
[LIBS="-l$2 $wk_otherlibs $LIBS"],
[$1="no"
AC_MSG_WARN([*** Linking library $2 failed (see config.log for details) ***])],
$wk_otherlibs)
fi
])

dnl ------------------------------------------------------------------------
dnl C compiler tests (CompHEP only)

AC_DEFUN(WK_PROG_COMPHEP_CC_WORKS,
[AC_CACHE_CHECK([whether this compiler works], 
wk_cv_prog_COMPHEP_CC_works,
[CC=$COMPHEP_PATH/$COMPHEP_CC
unset CFLAGS
unset CPPFLAGS
AC_LANG(C)
AC_COMPILE_IFELSE([AC_LANG_PROGRAM(,[[;]])],
wk_cv_prog_COMPHEP_CC_works=yes,
[AC_MSG_RESULT(no)
AC_MSG_FAILURE([Comphep C compiler failed])])
])
])


dnl ------------------------------------------------------------------------
dnl Further tools

dnl Test for GNU tar:
dnl
dnl WK_PROG_GNU_TAR()
AC_DEFUN(WK_PROG_GNU_TAR,
[AC_CACHE_CHECK([for GNU tar], [wk_cv_prog_GTAR],
[wk_cv_prog_GTAR=$GTAR
test -z "$wk_cv_prog_GTAR" && wk_cv_prog_GTAR=tar
if test -n "`$wk_cv_prog_GTAR --version 2>/dev/null | grep GNU`"; then
  :
elif test -n "`gtar --version 2>/dev/null | grep GNU`"; then
  wk_cv_prog_GTAR=gtar
else
  wk_cv_prog_GTAR=no
fi
])
GTAR=$wk_cv_prog_GTAR
])

dnl Test for GNU make:
dnl
dnl WK_PROG_GNU_MAKE()
AC_DEFUN(WK_PROG_GNU_MAKE,
[AC_CACHE_CHECK([for GNU make], [wk_cv_prog_GMAKE],
[wk_cv_prog_GMAKE="$GMAKE"  # User-supplied command
test -z "$wk_cv_prog_GMAKE" && wk_cv_prog_GMAKE="make"
if test -n "`$wk_cv_prog_GMAKE --version 2>/dev/null | grep GNU`"; then
  :
elif test -n "`gmake --version 2>/dev/null | grep GNU`"; then
  wk_cv_prog_GMAKE=gmake
else
  wk_cv_prog_GMAKE=no
fi
])
GMAKE=$wk_cv_prog_GMAKE
])


dnl Test whether Metapost works and can dump a .mem file
dnl
dnl WK_PROG_MPOST_WORKS()
AC_DEFUN(WK_PROG_MPOST_WORKS,
[AC_CACHE_CHECK([whether Metapost ($MPOST) works], [wk_cv_prog_mpost_works],
[touch conftest.mp
echo dump | $MPOST --ini conftest >/dev/null 2>&1
if test -f conftest.mem; then
  wk_cv_prog_mpost_works="yes"
else
  wk_cv_prog_mpost_works="no"
fi
rm -f conftest.mp conftest.mem conftest.log

if test $wk_cv_prog_mpost_works = "no"; then
  AC_MSG_ERROR([installation or configuration problem: METAPOST cannot create .mem files.])
fi
dnl AC_MSG_RESULT($wk_cv_prog_mpost_works)
])])


